package de.amberhome.materialtaptarget;

import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.view.View;

import anywheresoftware.b4a.BA;
import uk.co.samuelwall.materialtaptargetprompt.MaterialTapTargetPrompt;

@BA.Author("Markus Stipp")
@BA.ActivityObject
@BA.Version(1.10f)
@BA.DependsOn(values = {"MaterialTapTarget.aar", "material-tap-target-prompt-2.0.1.aar"})
@BA.ShortName("MaterialTapTargetBuilder")
public class MaterialTapTargetBuilderWrapper {

    @BA.Hide
    public MaterialTapTargetPrompt.Builder mBuilder;

    @BA.Hide
    // public MaterialTapTargetPrompt mPrompt;
    private String mEventName;
    private BA mBa;

    public void Initialize(final BA ba, String EventName) {
        mEventName = EventName.toLowerCase(ba.cul);
        mBa = ba;

        mBuilder = new MaterialTapTargetPrompt.Builder(ba.activity);

        mBuilder.setPromptStateChangeListener(new MaterialTapTargetPrompt.PromptStateChangeListener() {
            @Override
            public void onPromptStateChanged(MaterialTapTargetPrompt prompt, int state) {
                MaterialTargetPromptWrapper pw = new MaterialTargetPromptWrapper(prompt, mBa, mEventName);
                ba.raiseEventFromUI(prompt, mEventName + "promptstatechanged", pw, state);
            }
        });
    }

    /**
     *
     */
    public MaterialTapTargetBuilderWrapper BackButtonDismissEnabled(boolean Enable) {
        mBuilder.setBackButtonDismissEnabled(Enable);
        return this;
    }

    /**
     * Set the view for the prompt to focus on.
     */
    public MaterialTapTargetBuilderWrapper TargetView(View View) {
        mBuilder.setTarget(View);
        return this;
    }

    /**
     * Set the centre point as a screen position
     */
    public MaterialTapTargetBuilderWrapper TargetPos(int Left, int Top) {
        mBuilder.setTarget(Left, Top);
        return this;
    }

    /**
     * Change the view that is rendered as the target.
     * By default the view from TargetView(View) is rendered as the target.
     */
    public MaterialTapTargetBuilderWrapper TargetRenderView(View View) {
        mBuilder.setTargetRenderView(View);
        return this;
    }

    /**
     * Checks if the target is correctly set.
     */
    public boolean getIsTargetSet() {
        return mBuilder.isTargetSet();
    }

    /**
     * Set the primary text to the given string
     */
    public MaterialTapTargetBuilderWrapper PrimaryText(String Text) {
        mBuilder.setPrimaryText(Text);
        return this;
    }

    /**
     * Set the primary text font size.
     */
    public MaterialTapTargetBuilderWrapper PrimaryTextSize(float Size) {
        mBuilder.setPrimaryTextSize(Size);
        return this;
    }

    /**
     * Set the primary text color.
     */
    public MaterialTapTargetBuilderWrapper PrimaryTextColor(int Color) {
        mBuilder.setPrimaryTextColour(Color);
        return this;
    }

    /**
     * Sets the typeface used to display the primary text.
     */
    public MaterialTapTargetBuilderWrapper PrimaryTextTypeface(Typeface Typeface, int Style) {
        mBuilder.setPrimaryTextTypeface(Typeface, Style);
        return this;
    }

    /**
     * Set the secondary text.
     */
    public MaterialTapTargetBuilderWrapper SecondaryText(String Text) {
        mBuilder.setSecondaryText(Text);
        return this;
    }

    /**
     * Set the secondary text font size.
     */
    public MaterialTapTargetBuilderWrapper SecondaryTextSize(float Size) {
        mBuilder.setSecondaryTextSize(Size);
        return this;
    }

    /**
     * Set the secondary text colour.
     */
    public MaterialTapTargetBuilderWrapper SecondaryTextColor(int Color) {
        mBuilder.setSecondaryTextColour(Color);
        return this;
    }

    /**
     * Sets the typeface and style used to display the secondary text.
     */
    public MaterialTapTargetBuilderWrapper SecondaryTextTypeface(Typeface Typeface, int Style) {
        mBuilder.setSecondaryTextTypeface(Typeface, Style);
        return this;
    }

    /**
     * Set the text left and right padding.
     */
    public MaterialTapTargetBuilderWrapper TextPadding(float Padding) {
        mBuilder.setTextPadding(Padding);
        return this;
    }

    /**
     * Set the distance between the primary and secondary text.
     */
    public MaterialTapTargetBuilderWrapper TextSeparation(float Separation) {
        mBuilder.setTextSeparation(Separation);
        return this;
    }

    /**
     * Set the padding between the text and the focal point.
     */
    public MaterialTapTargetBuilderWrapper FocalPadding(float Padding) {
        mBuilder.setFocalPadding(Padding);
        return this;
    }

    /**
     * Enable/disable animation above target.
     * true by default
     */
    public MaterialTapTargetBuilderWrapper IdleAnimationEnabled(boolean Enabled) {
        mBuilder.setIdleAnimationEnabled(Enabled);
        return this;
    }

    /**
     * Set the icon to draw in the focal point.
     */
    public MaterialTapTargetBuilderWrapper Icon(Drawable Icon) {
        mBuilder.setIconDrawable(Icon);
        return this;
    }

    /**
     * Set the icon to draw in the focal point using the given resource name.
     */
    public MaterialTapTargetBuilderWrapper IconRes(String ResourceName) {
        mBuilder.setIcon(BA.applicationContext.getResources().getIdentifier(ResourceName, "drawable", BA.packageName));
        return this;
    }

    /**
     * Sets the color to use to tint the icon drawable.
     */
    public MaterialTapTargetBuilderWrapper IconColor(int Color) {
        mBuilder.setIconDrawableColourFilter(Color);
        return this;
    }

    /**
     * Set if the prompt should stop touch events on the focal point from passing
     * to underlying views. Default is false.
     */
    public MaterialTapTargetBuilderWrapper CaptureTouchEventOnFocal(boolean Capture) {
        mBuilder.setCaptureTouchEventOnFocal(Capture);
        return this;
    }

    /**
     * Set the max width that the primary and secondary text can be.
     */
    public MaterialTapTargetBuilderWrapper MaxTextWidth(float Width) {
        mBuilder.setMaxTextWidth(Width);
        return this;
    }

    /**
     * Set the background colour.
     */
    public MaterialTapTargetBuilderWrapper BackgroundColor(int Color) {
        mBuilder.setBackgroundColour(Color);
        return this;
    }

    /**
     * Set the focal point colour.
     */
    public MaterialTapTargetBuilderWrapper FocalColor(int Color) {
        mBuilder.setFocalColour(Color);
        return this;
    }

    /**
     * Set the focal point radius.
     */
    public MaterialTapTargetBuilderWrapper FocalRadius(float Radius) {
        mBuilder.setFocalRadius(Radius);
        return this;
    }

    /**
     * Set whether the prompt should dismiss itself when a touch event occurs outside the focal.
     * Default is true.
     */
    public MaterialTapTargetBuilderWrapper AutoDismiss(boolean AutoDismiss) {
        mBuilder.setAutoDismiss(AutoDismiss);
        return this;
    }

    /**
     * Set whether the prompt should finish itself when a touch event occurs inside the focal.
     * Default is true.
     */
    public MaterialTapTargetBuilderWrapper AutoFinish(boolean AutoFinish) {
        mBuilder.setAutoFinish(AutoFinish);
        return this;
    }

    /**
     * Set if the prompt should stop touch events outside the prompt from passing
     * to underlying views. Default is false.
     */
    public MaterialTapTargetBuilderWrapper CaptureTouchEventOutsidePrompt(boolean Capture) {
        mBuilder.setCaptureTouchEventOutsidePrompt(Capture);
        return this;
    }

    /**
     * Set the primary text horizontal layout gravity.
     * Default: Gravity.START
     */
    public MaterialTapTargetBuilderWrapper PrimaryTextGravity(int Gravity) {
        mBuilder.setPrimaryTextGravity(Gravity);
        return this;
    }

    /**
     * Set the secondary text horizontal layout gravity.
     * Default: Gravity.START
     */
    public MaterialTapTargetBuilderWrapper SecondaryTextGravity(int Gravity) {
        mBuilder.setSecondaryTextGravity(Gravity);
        return this;
    }

    /**
     * Set the view to clip the prompt to.
     *
     * Null can be used to stop the prompt being clipped to a view.
     */
    public MaterialTapTargetBuilderWrapper ClipToView(View View) {
        mBuilder.setClipToView(View);
        return this;
    }

    /**
     * Creates an MaterialTapTargetPrompt with the arguments supplied to this
     * builder.
     *
     * Calling this method does not display the prompt. If no additional
     * processing is needed, Show() may be called instead to both
     * create and display the prompt.
     *
     * Will return Null if a valid target has not been set or the primary text is Null.
     * To check that a valid target has been set call IsTargetSet.
     */
    public MaterialTargetPromptWrapper Create() {
        return new MaterialTargetPromptWrapper(mBuilder.create(), mBa, mEventName);
    }

    /**
     * Creates an MaterialTapTargetPrompt with the arguments supplied to this
     * builder.
     *
     * Calling this method does not display the prompt. If no additional
     * processing is needed, Show() may be called instead to both
     * create and display the prompt.
     *
     * Will return Null if a valid target has not been set or the primary text is Null.
     * To check that a valid target has been set call IsTargetSet.
     */
    public MaterialTargetPromptWrapper Show() {
        return new MaterialTargetPromptWrapper(mBuilder.show(), mBa, mEventName);
    }

}
